//
//  BTDataProcessVC.m
//  Unity-iPhone
//
//  Created by leon on 2021/2/3.
#import "BTDataProcess.h"
#import "IOSPlatformSDK.h"
#include "Game.h"

//#define NSLog(format, ...) printf("TIME:%s FILE:%s(%d行) FUNCTION:%s %s\n",__TIME__, [[[NSString stringWithUTF8String:__FILE__] lastPathComponent] UTF8String], __LINE__, __PRETTY_FUNCTION__, [[NSString stringWithFormat:(format), ##__VA_ARGS__] UTF8String])
@interface BTDataProcess ()<NSURLSessionDataDelegate>
{
@private
Game *game;
Game *viceDeviceGame;
}
@property (strong, nonatomic)NSMutableArray  * deviceArray;//蓝牙外设数据源
@property (nonatomic,strong)IOSPlatformSDK * sdk;//ios_sdk
@property(nonatomic,weak)NSTimer * timer;//定时请求设备状态
@property(nonatomic,assign)BOOL gameModel;//鞋子是否开启游戏模式
@property(nonatomic,strong)NSDate * firstTime;//当局游戏时间
//当局游戏跳起、下蹲次数
@property(assign,nonatomic)int initial_jump_count;
@property(assign,nonatomic)int initial_down_count;
@end

@implementation BTDataProcess

#pragma mark ===============================================>> 静态的初始化方法  init的时候（建立蓝牙中心管理类，设置代理）
//单例静态
static BTDataProcess* instance = nil;
+(instancetype)sharedInstance{
     return [[self alloc] init];
}
+ (instancetype)allocWithZone:(struct _NSZone *)zone{
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
      instance = [super allocWithZone:zone];
  });
  return instance;
}
- (instancetype)init{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [super init];
        //后台模式
        self.isBackGround = NO;
        //默认游戏模式关
        self.gameModel = NO;
    });
    return instance;
}

#pragma mark ===============================================>> public method
//趣动 ==>> 断开蓝牙
-(void)disConnedctBle{
    NSLog(@"趣动 call 断开蓝牙");
    [LEManager cancelPeripheralConnection];
}

//每重新开始一局游戏
-(void)gameStartInitData{
    //当局游戏时间
    self.firstTime = [NSDate date];
    //当局动作数据
    self.initial_down_count = 0;
    self.initial_jump_count = 0;
    //游戏模式 ==>> 回调剑波sdk数据
    self.gameModel = YES;
    //读写到特征后 添加测试弹窗
    dispatch_async(dispatch_get_main_queue(), ^{
//        [DebugViewInstance inittestLabel];
    });
}

//每结束一局游戏
-(void)gameEndInitData{
    self.gameModel = NO;
}

//监听通知震动
-(void)vibrationAction:(int)type duration:(int)duration{
    [BTDataSendInstance vibration:(DEVICE_TYPE)type duration:duration];
}

#pragma mark ============================================================== leon Ble manager
//测试自己封装的蓝牙类
-(void)initCBCentralManager{

    if (LEManager.centralManager == nil && LEManager.peripheral == nil && LEManager.vicePeripheral == nil){
        [LEManager initCBCentralManager];
        NSLog(@" ====================>> 未初始化 centralManager 直接开始初始化");
    }else{
        [LEManager scanForPeripheralsWithServices:nil options:nil];
        NSLog(@" ====================>> 已初始化 centralManager 直接开扫描");
    }
    
    LEManager.stateUpdateBlock = ^(CBCentralManager *  central){
        switch(central.state){
            case 0:
                NSLog(@"当前的蓝牙状态 ===>> CBCentralManagerStateUnknown");
                break;
            case 1:
                NSLog(@"当前的蓝牙状态 ===>> CBCentralManagerStateResetting");
                break;
            case 2:
                NSLog(@"当前的蓝牙状态 ===>> CBCentralManagerStateUnsupported");
                break;
            case 3:
                NSLog(@"当前的蓝牙状态 ===>> CBCentralManagerStateUnauthorized");
                break;
            case 4:{
                NSLog(@"当前的蓝牙状态 ===>> 蓝牙已关闭");
                if (LEManager.peripheral!=nil){
                    
                    [self.sdk bridgingDeviceAction:DEVICETYPE_MAIN
                                                name:LEManager.peripheral.name
                                             address:LEManager.peripheral.identifier.UUIDString
                                              status:CONNECT_LOST
                                         electricity:0];
                    
                }
                if (LEManager.vicePeripheral!=nil){
                    
                    [self.sdk bridgingDeviceAction:DEVICETYPE_VICE
                                                name:LEManager.vicePeripheral.name
                                             address:LEManager.vicePeripheral.identifier.UUIDString
                                              status:CONNECT_LOST
                                         electricity:0];
                }
                
            }
                break;
            case 5:{
                NSLog(@"当前的蓝牙状态 ===>> 蓝牙已开启");//蓝牙已开启
                [LEManager scanForPeripheralsWithServices:nil options:nil];
            }
                break;
            default:
                break;
        }
        
    };
    
    [self.deviceArray removeAllObjects];
    //发现蓝牙
    LEManager.discoverPeripheralBlock = ^(CBCentralManager * _Nonnull central, CBPeripheral * _Nonnull peripheral, NSDictionary * _Nonnull advertisementData, NSNumber * _Nonnull RSSI){
        //        NSLog(@"扫描发现蓝牙设备advertisementData =  %@",advertisementData);
        NSString *advertiseName = [advertisementData objectForKey:CBAdvertisementDataLocalNameKey];

                peripheral.advertiseName = advertiseName;

                if ([advertiseName hasPrefix:@"SH_"]){//SH_DANCE BLE_LOOP  Shoes_4530 Shoes_2A74
                    
//                    NSLog(@"主设备扫描发现有效的蓝牙设备 =============== %@   identifier == %@",peripheral.name,peripheral.identifier.UUIDString);

                    if (self.deviceArray.count == 0){//数据源为0
                        
                            [self.deviceArray addObject:peripheral];
                            //判断扫描到外设之后是否要链接蓝牙
                            [self ifAppJumpWithIdentifier:peripheral];
                        
                    }else{
                        
                            BOOL isExist = NO;
                            for (int i = 0; i < self.deviceArray.count; i++){
                                CBPeripheral *per = [self.deviceArray objectAtIndex:i];
                                if ([per.identifier.UUIDString isEqualToString:peripheral.identifier.UUIDString]){
                                    isExist = YES;
                                    [self.deviceArray replaceObjectAtIndex:i withObject:peripheral];
                                }
                            }
                        
                            if (!isExist){
                                [self.self.deviceArray addObject:peripheral];
                                //判断扫描到外设之后是否要链接蓝牙
                                [self ifAppJumpWithIdentifier:peripheral];
                            }
                    }
                    
                    //数据回调给 搜索蓝牙弹窗
                    if (self.deviceArrBLock != nil){
                        self.deviceArrBLock(self.deviceArray);
                    }

            }
        
     };

}

#pragma mark ===============================================>>  主设备蓝牙数据
//是否是app跳转传参 区分app跳转和弹窗搜索 type=0的时候才会执行该方法 type=0的时候才会执行该方法 当时弹框的时候type==1 不执行该方法
-(void)ifAppJumpWithIdentifier:(CBPeripheral*)peripheral{
    
    if (self.macAddress!=nil && [peripheral.identifier.UUIDString isEqualToString:self.macAddress]){//app跳转过来 主动链接蓝牙
        [self connectPeripheral:peripheral];
    }
    
}

#pragma mark ===============================================>> 链接蓝牙设备公用方法  可外部操作
//app跳转或选中tableview 链接蓝牙
-(void)connectPeripheral:(CBPeripheral*)peripheral{
    
    if(instance!=NULL && peripheral!=NULL){
       
        self.sdk = [IOSPlatformSDK sharedInstance];//蓝牙数据经过sdk数据转换后 ios call unity
        DebugViewInstance.currentTS = 1000;
        NSLog(@"BTDataProcess connectPeripheral self.deviceType ==>> %ld  self.game_type ==>> %d   mac ==>>  %@",(long)self.deviceType,self.game_type,self.macAddress);
        
        //初始化运轨sdk
        if (self.deviceType==DEVICETYPE_MAIN){
            LEManager.peripheral = peripheral;
            game = new Game(self.game_type);
        }else if (self.deviceType == DEVICETYPE_VICE){
            LEManager.vicePeripheral = peripheral;
            viceDeviceGame = new Game(self.game_type);
        }
        
        //根据ios_sdk传入deviceType 链接 主/副 设备
        [self connectBLEManagerData:peripheral deviceType:self.deviceType];
    
    }
    
}

//链接主副设备 + 绑定特征
-(void)connectBLEManagerData:(CBPeripheral*)peripheral deviceType:(DEVICE_TYPE)deviceType{
    
    //链接动画
    [[AnimationView shanreAnimationView] addAnimation];
    
    //链接蓝牙 此时 LEManager.peripheral || LEManager.vicePeripheral 还未init
    [LEManager connectPeripheral:peripheral options:nil];
    //发现服务和特征
    LEManager.discoverCharacteristicsBlock = ^(CBPeripheral * _Nonnull peripheral, CBService * _Nonnull service, NSArray * _Nonnull characteristics, NSError * _Nonnull error){
        
        if (peripheral==LEManager.peripheral&&deviceType==DEVICETYPE_MAIN){//主设备
            
            for (CBCharacteristic * cha in service.characteristics){
                
                if (cha.properties == 12){//写
                    
                    LEManager.write = cha;
                    [self initGameAction:deviceType];

                }else if (cha.properties == 16){//读
                    
                    LEManager.read  = cha;
                    [LEManager.peripheral readValueForCharacteristic:cha];
                    [LEManager.peripheral setNotifyValue:YES forCharacteristic:cha];
                    
                }

            }

        }else if (peripheral==LEManager.vicePeripheral&&deviceType==DEVICETYPE_VICE){//副设备

            for (CBCharacteristic * cha in service.characteristics){
                
                if (cha.properties == 12){//写

                    LEManager.viceWrite = cha;
                    [self initGameAction:deviceType];

                }else if (cha.properties == 16){//读

                    LEManager.viceRead  = cha;
                    [LEManager.vicePeripheral readValueForCharacteristic:cha];
                    [LEManager.vicePeripheral setNotifyValue:YES forCharacteristic:cha];

                }
            }
        }
    };
    
    //读取特征的报文数据
    LEManager.readValueForCharacteristicBlock = ^(CBPeripheral * _Nonnull peripheral, CBCharacteristic * _Nonnull characteristic, NSData * _Nonnull value, NSError * _Nonnull error, DEVICE_TYPE type){
        if ([characteristic.UUID.UUIDString isEqualToString:@"6E400003-B5A3-F393-E0A9-E50E24DCCA9E"]){//判断是不是我们设备的特征值
//            NSLog(@"characteristic.UUID.UUIDString = %@",characteristic.UUID.UUIDString);
                [self verifyData:characteristic deviceType:type];
                [peripheral readRSSI];
         }
    };

    //写入数据的回调 暂时用不到
    LEManager.writeToCharacteristicBlock = ^(CBPeripheral * _Nonnull peripheral,CBCharacteristic * _Nonnull characteristic, NSError * _Nonnull error, DEVICE_TYPE type){
    };
    
    //蓝牙更新RSSI
    LEManager.rssiBlock = ^(NSNumber * _Nonnull RSSI){
        DebugViewInstance.RSSI = RSSI;
//        NSLog(@"self.RSSI  = %@",self.RSSI);
    };
    
    //蓝牙连接成功
    LEManager.successfulBlock = ^(CBPeripheral * _Nonnull peripheral){
            [[AnimationView shanreAnimationView] stopAnimation];
    };

    //蓝牙连接失败
    LEManager.connectFailureBlock = ^(NSError * _Nonnull error){
        NSLog(@"蓝牙链接失败");
    };

    //蓝牙连接丢失
    LEManager.disConnectBlock = ^(CBPeripheral * _Nonnull peripheral, NSError * _Nonnull error){
        
        if (LEManager.peripheral!=nil&&peripheral == LEManager.peripheral){
            
            [self.sdk bridgingDeviceAction:DEVICETYPE_MAIN
                                        name:LEManager.peripheral.name
                                     address:LEManager.peripheral.identifier.UUIDString
                                      status:CONNECT_LOST
                                 electricity:0];
            
        }
        
        if (LEManager.vicePeripheral!=nil&&peripheral == LEManager.vicePeripheral){
            
            [self.sdk bridgingDeviceAction:DEVICETYPE_VICE
                                        name:LEManager.vicePeripheral.name
                                     address:LEManager.vicePeripheral.identifier.UUIDString
                                      status:CONNECT_LOST
                                 electricity:0];
        }
        
        NSLog(@"蓝牙链接丢失");
        //断线重连
        [self initCBCentralManager];
        
    };
    
}


#pragma mark ===============================================>> 读取特征后 鞋子打开游戏模式 & 激光开关 &查询鞋子蓝牙状态 & 主设备初始化缓存游戏步数
-(void)initGameAction:(DEVICE_TYPE)deviceType{
    
    //开启游戏模式&查询设备信息 初始化步数数据
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2.0 * NSEC_PER_SEC)), dispatch_get_main_queue(),^{
        [self timingDataTransmission:@"dispatch_after"];
    });
    
    //定时开启游戏模式 & 查询设备信息 & 开启激光检测下蹲
    if (self.timer!=nil){
        [self.timer invalidate];
        self.timer = nil;
    }
    self.timer = [NSTimer scheduledTimerWithTimeInterval:10 target:self selector:@selector(timingDataTransmission:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:self.timer forMode:NSDefaultRunLoopMode];

}

//定时发送数据
-(void)timingDataTransmission:(NSString*)style{
    
    if (self.isBackGround){//程序在后台
        
    }else{//程序在前台
//        NSLog(@"定时发送 == %@",style);
        [BTDataSendInstance startGameModel];
    }
    
    [BTDataSendInstance queryDevideInfo];

}


#pragma mark ===============================================>> 校验 & 报文数据解析
-(void)verifyData:(CBCharacteristic*)characteristic deviceType:(DEVICE_TYPE)deviceType{
    
     if (characteristic.value.length>3){
         
         //带cmd位的有效数据  计算校验位
         if ([AlgorithmTool verificationRusult:characteristic.value]==YES){
             
             [self analysisCharacteristic:characteristic.value deviceType:deviceType];
                              
         }
         
     }else{
         
         NSLog(@"无效报文");
         
     }
     
}

-(void)analysisCharacteristic:(NSData*)characteristic deviceType:(DEVICE_TYPE)deviceType{
        
//        NSLog(@"接收到的数据data = %@",characteristic);
    int frameHead = [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(0, 1)]];//帧头数据aa
    int messageLength = [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(1, 1)]];//报文长度
    int messageLengthNegation = ~messageLength;//报文长度取反
    int dataType = [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(3, 1)]];//cmd类型
    
      if (dataType == 4){//获取主动推过来的数据
            //右脚坐标数据
           int right_X = [AlgorithmTool dataToSwapBigIntToHost:characteristic andRange:NSMakeRange(4, 4)];
           int right_Y = [AlgorithmTool dataToSwapBigIntToHost:characteristic andRange:NSMakeRange(8, 4)];
           int right_Z = [AlgorithmTool dataToSwapBigIntToHost:characteristic andRange:NSMakeRange(12, 4)];
           int  right_pos[3] = {right_X,right_Y,right_Z};
          //左脚坐标数据
           int left_X = [AlgorithmTool dataToSwapBigIntToHost:characteristic andRange:NSMakeRange(16, 4)];
           int left_Y = [AlgorithmTool dataToSwapBigIntToHost:characteristic andRange:NSMakeRange(20, 4)];
           int left_Z = [AlgorithmTool dataToSwapBigIntToHost:characteristic andRange:NSMakeRange(24, 4)];
           int  left_pos[3] = {left_X,left_Y,left_Z};
           //右脚姿势数据
           short righrPosture_X = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(28, 2)];
           short righrPosture_Y = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(30, 2)];
           short righrPosture_Z = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(32, 2)];
           int  right_att[3] = {righrPosture_X,righrPosture_Y,righrPosture_Z};
           //左脚姿势数据
           short leftPosture_X = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(34, 2)];
           short leftPosture_Y = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(36, 2)];
           short leftPosture_Z = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(38, 2)];
           int  left_att[3] = {leftPosture_X,leftPosture_Y,leftPosture_Z};
           //右脚三维数据
           short righrAcc_x = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(40, 2)];
           short righrAcc_Y = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(42, 2)];
           short righrAcc_z = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(44, 2)];
           int  righrAcc[3] = {righrAcc_x,righrAcc_Y,righrAcc_z};
           //左脚三维数据
           short leftAcc_x = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(46, 2)];
           short leftAcc_y = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(48, 2)];
           short leftAcc_z = [AlgorithmTool dataToSwapBigShortToHost:characteristic andRange:NSMakeRange(50, 2)];
           int  left_acc[3] = {leftAcc_x,leftAcc_y,leftAcc_z};
           //左脚、右脚、蹲 、跳四个动作信息数据
           int actionInformation =  [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(52, 1)]];;//0c 16进制字符串
                       int girlShoes  = actionInformation & 16;
                       if (girlShoes!=0){
                           girlShoes=1;
                       }else{
                           girlShoes=0;
                       }
                       int rightZupt  = actionInformation & 8;
                       if (rightZupt!=0){
                           rightZupt=1;
                       }else{
                           rightZupt=0;
                       }
                       int leftZupt = actionInformation & 4;
                       if (leftZupt!=0){
                            leftZupt=1;
                        }else{
                            leftZupt=0;
                        }
                       int down  = actionInformation & 2;
                       if (down!=0){
                            down=1;
                        }else{
                            down=0;
                        }
                       int jump  = actionInformation & 1;
                       if (jump!=0){
                            jump=1;
                        }else{
                            jump=0;
                        }
          
            int rssi = [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(53, 1)]];//rssi 信号强度
            int ts = [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(54, 1)]];;//ts 时间戳
            int right_press = [AlgorithmTool dataToInt:[characteristic subdataWithRange:NSMakeRange(55, 2)]];//3 29新增 右鞋压力
            int left_press = [AlgorithmTool dataToInt:[characteristic subdataWithRange:NSMakeRange(57, 2)]];//3 29新增 左鞋压力
            int frameCheck =  [AlgorithmTool dataToChar:[characteristic subdataWithRange:NSMakeRange(59, 1)]];//校验位
        
               if (deviceType==DEVICETYPE_MAIN){//主设备
//                   NSLog(@"主设备 接收到 鞋子动作数据 data = %@",characteristic);
                   //调取鞋子SDK
                   [self shoseSDKRight_pos:right_pos Right_att:right_att Right_acc:righrAcc LeftPos:left_pos Left_att:left_att Left_acc:left_acc ts:ts rightZupt:rightZupt leftZupt:leftZupt jump:jump down:down rssi:rssi girlShoes:girlShoes right_press:right_press left_press:left_press characteristic:characteristic];
                   
               }else if (deviceType==DEVICETYPE_VICE){//副设备动作数据
                   //调取鞋子SDK
                   [self viceShoseSDKRight_pos:right_pos Right_att:right_att Right_acc:righrAcc LeftPos:left_pos Left_att:left_att Left_acc:left_acc ts:ts rightZupt:rightZupt leftZupt:leftZupt jump:jump down:down rssi:rssi girlShoes:girlShoes right_press:right_press left_press:left_press characteristic:characteristic];
               }
              
        }else if (dataType == -95 && characteristic.length == 26){//获取的是查询的数据 char-->int a1 = -95
           //A1+1 >> 1: 设备数据(左鞋,右鞋)  aa 14 eb a1 01 64 00 1a 0000981d 64 00 17 00000000 f9
                int leftElectricity = [AlgorithmTool dataToInt:[characteristic subdataWithRange:NSMakeRange(5, 1)]];
                int rightElectricity = [AlgorithmTool dataToInt:[characteristic subdataWithRange:NSMakeRange(15, 1)]];//龙哥新协议
                
                if (deviceType==DEVICETYPE_MAIN && LEManager.peripheral!=nil){//主设备
//                    NSLog(@"主设备 接收到 鞋子硬件数据 data 26 = %@  电量 %d %d %@ %@",characteristic,leftElectricity,rightElectricity,LEManager.peripheral.name,LEManager.peripheral.identifier.UUIDString);
                    //主 设备定时器 60秒调一次   /******************ios call unity*****************/
                     [self.sdk bridgingDeviceAction:DEVICETYPE_MAIN
                                                 name:LEManager.peripheral.name
                                              address:LEManager.peripheral.identifier.UUIDString
                                               status:CONNECT_ED
                                          electricity:leftElectricity<=rightElectricity?leftElectricity:rightElectricity];
             
                }else if (deviceType==DEVICETYPE_VICE && LEManager.vicePeripheral!=nil){//副设备
//                    NSLog(@"副设备 接收到 鞋子硬件数据 26 = %@  电量 %d %d",characteristic,leftElectricity,rightElectricity);
                    //副 设备定时器 60秒调一次   /******************ios call unity*****************/
                     [self.sdk bridgingDeviceAction:DEVICETYPE_VICE
                                                 name:LEManager.vicePeripheral.name
                                              address:LEManager.vicePeripheral.identifier.UUIDString
                                               status:CONNECT_ED
                                          electricity:leftElectricity<=rightElectricity?leftElectricity:rightElectricity];
                    
                }
        }
}

#pragma mark ===============================================>> 主设备 报文数据解析后 -- 调用蓝牙鞋子SDK装换数据 -- ios call unity 更新游戏动作
-(void)shoseSDKRight_pos:(int[3])right_pos
               Right_att:(int[3])right_att
               Right_acc:(int[3])right_acc
                 LeftPos:(int[3])left_pos
                Left_att:(int[3])left_att
                Left_acc:(int[3])left_acc
                      ts:(int)ts
               rightZupt:(int)rightZupt
                leftZupt:(int)leftZupt
                    jump:(int)jump
                    down:(int)down
                    rssi:(int)rssi
               girlShoes:(int)girlShoes
             right_press:(int)right_press
              left_press:(int)left_press
          characteristic:(NSData*)characteristic{
    
    /********************初始化 运动轨迹算法 SDK *****************/
    //游戏数据
    NSData * validData= [characteristic subdataWithRange:NSMakeRange(4, characteristic.length-4)];
    Byte * buff = (Byte*)[validData bytes];
    game->GameProcessBuf(buff, (int)validData.length);
    
    int length = 4;
    int result[length];
    game->getGameResult(result);
    //   NSLog(@"result == %d %d %d %d \n",result[0],result[1],result[2],result[3]);

    /******************步频数据处理 + 回调*****************/
    int leftStepStatus =  game->getStepStatus(LEFT_FOOT);
    int leftStepFreq =  game->getStepFreq(LEFT_FOOT);
    int leftStepCount =  game->getStepCount(LEFT_FOOT);
    int rightStepStatus =  game->getStepStatus(RIGHT_FOOT);
    int rightStepFreq =  game->getStepFreq(RIGHT_FOOT);
    int rightStepCount =  game->getStepCount(RIGHT_FOOT);
        
    /******************左右脚动作数据处理 + 回调*****************/
    int motionLeft = result[0];//左脚的动作
    int motionRight = result[1];//右脚的动作
    int motionJump = result[2];//jump
    int motionDown = result[3];//down
//    if (motionLeft==-1 && motionRight == -1&& motionJump == -1&& motionDown == -1){//无效动作
//        return;
//    }else{
        //跳跃和蹲下的动作 双脚是同步的
        if (motionJump == MOTION_JUMP){
            motionLeft = motionJump;
            motionRight = motionJump;
            NSLog(@" 主设备 ================================== 跳起来 ");
        }
    
        if (motionDown == MOTION_DOWN){
            motionLeft = motionDown;
            motionRight = motionDown;
            NSLog(@" 主设备 =================================== 蹲下去 ");
        }
//    }
    
    /****************** ios call unity*****************/
        if (self.gameModel==YES){//当前是游戏模式

            //跑酷需要回调步频
            if (self.game_type == GAME_TYPE_RUN){
                [self.sdk bridgingStepAction:DEVICETYPE_MAIN leftStatus:leftStepStatus rightStatus:rightStepStatus leftFrag:leftStepFreq rightFrag:rightStepFreq];
            }
            //左右脚动作
            [self.sdk bridgingMotionAction:DEVICETYPE_MAIN left:motionLeft right:motionRight];

        }else{//当前是交互模式

            int interaction = game->getInteractionCMD();
//            NSLog(@"主设备 当前的脚步交互动作 ===== >> %d",interaction);
            [self.sdk bridgingInteraction:DEVICETYPE_MAIN code:interaction];
            if (self.getInteractionBlock){
                self.getInteractionBlock(interaction);
            }
        }
    
    /******************数据回调缓存*****************/
    //标记初始值
    if (self.initial_jump_count==0){
        self.initial_jump_count = game->getGameJumpCount();
    }
    if (self.initial_down_count==0) {
        self.initial_down_count = game->getGameDownCount();
    }
    //总数
    int total_jump_count = game->getGameJumpCount();
    int total_dump_count = game->getGameDownCount();
    //当局游戏次数
    int current_jump_count = total_jump_count - self.initial_jump_count;
    int current_dump_count = total_dump_count - self.initial_down_count;
    //数据回调缓存
    [self cacheAllMotionCount:current_jump_count crouchCount:current_dump_count stepCount:leftStepCount + rightStepCount];
    
    /********************剑波 & 威严 要的图表数据 *****************/
    NSString * aString = [NSString stringWithUTF8String:game->getGameDataStr().c_str()];
    if (DebugViewInstance.RSSI==nil){
        DebugViewInstance.RSSI = 0;
    }
     aString = [NSString stringWithFormat:@"%@,%@",aString,DebugViewInstance.RSSI];
    DebugViewInstance.tempStepString = [NSString stringWithFormat:@"%@\n",aString];
    if (DebugViewInstance.tempStepString!=nil && DebugViewInstance.tempStepString.length>1){
        [DebugViewInstance writeFileToplist];
    }
  
    /********************调试窗数据 *****************/
    NSArray *aArray = [DebugViewInstance.tempStepString componentsSeparatedByString:@","];
//    dispatch_async(dispatch_get_main_queue(), ^{
        if (DebugViewInstance.testLabel!=nil){
            DebugViewInstance.testLabel.text = [NSString stringWithFormat:@"right:[%d,%d,%d] [%d,%d,%d] [%d,%d,%d] \nleft:[%d,%d,%d] [%d,%d,%d] [%d,%d,%d] \nZupt:[%d,%d]  press:[%d,%d] \nj&d[%d %d]  rssi:[%d %@] ts:%d \n脚步: %@ %@\n步频: %d %d ",right_pos[0],right_pos[1],right_pos[2],right_att[0],right_att[1],right_att[2],right_acc[0],right_acc[1],right_acc[2],left_pos[0],left_pos[1],left_pos[2],left_att[0],left_att[1],left_att[2],left_acc[0],left_acc[1],left_acc[2],rightZupt,leftZupt,right_press,left_press,jump,down,rssi,DebugViewInstance.RSSI,ts,aArray[aArray.count-2],aArray.lastObject,leftStepFreq,rightStepFreq];
        }
//    });
    
    /********************颖嘉+的ts丢包数据 *****************/
    NSString * tsString =  aArray[aArray.count-4];
    int tsInt = [tsString  intValue];
    int total;
    if (DebugViewInstance.currentTS != 1000){//颖嘉初始值未知 ios初始值==1000
        //1....255
        if (tsInt > DebugViewInstance.currentTS){
             total = tsInt - DebugViewInstance.currentTS - 1;
        }
        //255...1
        if (tsInt < DebugViewInstance.currentTS){
            if (tsInt ==0) {
                total = (255-DebugViewInstance.currentTS) + tsInt + 1 - 1;
            }else{
                total = (255-DebugViewInstance.currentTS) + tsInt - 1;
            }
        }
        if (total > 20){
            dispatch_async(dispatch_get_main_queue(), ^{
                AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
                [PopupView showCusHUDA:[NSString stringWithFormat:@"丢包数: %d",total]];
            });
        }
    }
//    NSLog(@"tsInt = %d  currentTS = %d total = %d",tsInt,self.currentTS,total);
    DebugViewInstance.currentTS = tsInt;
    
}

#pragma mark ===============================================>>副设备 报文数据解析后 -- 调用蓝牙鞋子SDK装换数据 -- ios call unity 更新游戏动作
-(void)viceShoseSDKRight_pos:(int[3])right_pos
                   Right_att:(int[3])right_att
                   Right_acc:(int[3])right_acc
                     LeftPos:(int[3])left_pos
                    Left_att:(int[3])left_att
                    Left_acc:(int[3])left_acc
                          ts:(int)ts
                   rightZupt:(int)rightZupt
                    leftZupt:(int)leftZupt
                        jump:(int)jump
                        down:(int)down
                        rssi:(int)rssi
                   girlShoes:(int)girlShoes
                 right_press:(int)right_press
                  left_press:(int)left_press
              characteristic:(NSData*)characteristic{

    /********************初始化 运动轨迹算法 SDK *****************/
    //游戏数据
    NSData * validData= [characteristic subdataWithRange:NSMakeRange(4, characteristic.length-4)];
    Byte * buff = (Byte*)[validData bytes];
    viceDeviceGame->GameProcessBuf(buff, (int)validData.length);
    
    int length = 4;
    int result[length];
    viceDeviceGame->getGameResult(result);
    //   NSLog(@"result == %d %d %d %d \n",result[0],result[1],result[2],result[3]);

    /********************蓝牙sdk返回的鞋子 状态 步频 步数*****************/
    int leftStepStatus =  game->getStepStatus(LEFT_FOOT);
    int leftStepFreq =  game->getStepFreq(LEFT_FOOT);
    int leftStepCount =  game->getStepCount(LEFT_FOOT);
    int rightStepStatus =  game->getStepStatus(RIGHT_FOOT);
    int rightStepFreq =  game->getStepFreq(RIGHT_FOOT);
    int rightStepCount =  game->getStepCount(RIGHT_FOOT);
    
    /********************蓝牙sdk返回的鞋子 动作*****************/
    int motionLeft = result[0];//左脚的动作
    int motionRight = result[1];//右脚的动作
    int ivce_motionJump = result[2];//jump
    int ivce_motionDown = result[3];//down
    
//    if (motionLeft==-1 && motionRight == -1&& ivce_motionJump == -1&& ivce_motionDown == -1){//无效动作
//        return;
//    }else{
        //跳跃和蹲下的动作 双脚是同步的
        if (ivce_motionJump == MOTION_JUMP){
            motionLeft = ivce_motionJump;
            motionRight = ivce_motionJump;
            NSLog(@"副设备 ======================================== 跳起来");
        }
        if (ivce_motionDown == MOTION_DOWN){
            motionLeft = ivce_motionDown;
            motionRight = ivce_motionDown;
            NSLog(@"副设备 ========================================= 蹲下去");
        }
        
        //ios call unity
        if (self.game_type == GAME_TYPE_RUN){//跑酷才需要回调
            [self.sdk bridgingStepAction:DEVICETYPE_VICE leftStatus:leftStepStatus rightStatus:rightStepStatus leftFrag:leftStepFreq rightFrag:rightStepFreq];//副设备左右脚速度 步频
        }
        //ios call unity
        [self.sdk bridgingMotionAction:DEVICETYPE_VICE left:motionLeft right:motionRight];//副设备左右脚动作


}

#pragma mark ===============================================>> 懒加载 -- 蓝牙设备数据源
-(NSMutableArray*)deviceArray{
    if (!_deviceArray){
        _deviceArray = [NSMutableArray new];
    }
    return _deviceArray;;
}

//数据存入plist
//玩游戏过程中缓存 动作数据 和时间戳
-(void)cacheAllMotionCount:(int)jump_count crouchCount:(int)crouch_count stepCount:(int)step_count{
    
    //游戏持续时间
    NSTimeInterval durationTime = [[NSDate date] timeIntervalSinceDate:self.firstTime];//游戏总共经历的时长 秒数
    //缓存pilst文件
    NSMutableDictionary * bleDict = [NSMutableDictionary new];
    [bleDict setObject:[NSNumber numberWithInt:durationTime] forKey:@"duration"];
    [bleDict setObject:[NSNumber numberWithInt:jump_count] forKey:@"jump"];
    [bleDict setObject:[NSNumber numberWithInt:crouch_count] forKey:@"crouch"];
    [bleDict setObject:[NSNumber numberWithInt:step_count] forKey:@"step"];
    NSDictionary * dict = [NSDictionary dictionaryWithDictionary:bleDict];
    //归档缓存
    [IOS_NSUSERDEFAULT setObject:dict forKey:IOSSDK_BLESDKDATA];
    [IOS_NSUSERDEFAULT synchronize];
    
}

@end


